@@ -13,7 +13,7 @@
 #include <afsconfig.h>
 #include "../afs/param.h"
 
-RCSID("$Header: /cvs/openafs/src/afs/IRIX/osi_vfsops.c,v 1.4.4.2 2002/01/20 08:20:56 shadow Exp $");
+RCSID("$Header: /cvs/openafs/src/afs/IRIX/osi_vfsops.c,v 1.4.4.3 2002/04/24 01:41:32 zacheiss Exp $");
 
 #include "../afs/sysincludes.h"	/* Standard vendor system headers */
 #include "../afs/afsincludes.h"	/* Afs-based standard headers */
@@ -242,7 +242,7 @@ cred_t *cr;
      * rootvp gets lots of ref counts
      */
     if (rootvp) {
-	tvc = (struct vcache *)rootvp;
+	tvc = VTOAFS(rootvp);
         if (tvc->opens || CheckLock(&tvc->lock) || LockWaiters(&tvc->lock)) {
 	    ReleaseWriteLock(&afs_xvcache);
 	    return EBUSY;
@@ -290,13 +290,13 @@ afs_root (OSI_VFS_ARG(afsp), avpp)
     }
     if (tvp) {
 	int s;
-	VN_HOLD((struct vnode *)tvp);
-	s = VN_LOCK((struct vnode *)tvp);
-	tvp->v.v_flag |= VROOT;	
-	VN_UNLOCK((struct vnode *)tvp, s);
+	VN_HOLD(AFSTOV(tvp));
+	s = VN_LOCK(AFSTOV(tvp));
+	AFSTOV(tvp)->v_flag |= VROOT;	
+	VN_UNLOCK(AFSTOV(tvp), s);
 
 	afs_globalVFS = afsp;
-	*avpp = (struct vnode *) tvp;
+	*avpp = AFSTOV(tvp);
     }
 
     afs_Trace2(afs_iclSetp, CM_TRACE_VFSROOT, ICL_TYPE_POINTER, *avpp,
