@@ -23,11 +23,12 @@
 #include <afsconfig.h>
 #include "../afs/param.h"
 
-RCSID("$Header: /cvs/openafs/src/afs/LINUX/osi_vnodeops.c,v 1.29.2.3 2002/01/20 09:01:51 shadow Exp $");
+RCSID("$Header: /cvs/openafs/src/afs/LINUX/osi_vnodeops.c,v 1.29.2.11 2002/05/06 16:21:42 shadow Exp $");
 
 #include "../afs/sysincludes.h"
 #include "../afs/afsincludes.h"
 #include "../afs/afs_stats.h"
+#include "../afs/afs_osidnlc.h"
 #include "../h/mm.h"
 #include "../h/pagemap.h"
 #if defined(AFS_LINUX24_ENV)
@@ -60,7 +61,7 @@ static ssize_t afs_linux_read(struct file *fp, char *buf, size_t count,
 			      loff_t *offp)
 {
     ssize_t code;
-    struct vcache *vcp = (struct vcache*)fp->f_dentry->d_inode;
+    struct vcache *vcp = ITOAFS(fp->f_dentry->d_inode);
     cred_t *credp = crref();
     struct vrequest treq;
 
@@ -104,7 +105,7 @@ static ssize_t afs_linux_write(struct file *fp, const char *buf, size_t count,
 {
     ssize_t code = 0;
     int code2;
-    struct vcache *vcp = (struct vcache *)fp->f_dentry->d_inode;
+    struct vcache *vcp = ITOAFS(fp->f_dentry->d_inode);
     struct vrequest treq;
     cred_t *credp = crref();
 
@@ -157,7 +158,7 @@ static int afs_linux_readdir(struct file *fp,
 			     void *dirbuf, filldir_t filldir)
 {
     extern struct DirEntry * afs_dir_GetBlob();
-    struct vcache *avc = (struct vcache*)FILE_INODE(fp);
+    struct vcache *avc = ITOAFS(FILE_INODE(fp));
     struct vrequest treq;
     register struct dcache *tdc;
     int code;
@@ -314,7 +315,7 @@ void afs_linux_vma_close(struct vm_area_struct *vmap)
     if (!vmap->vm_file)
 	return;
 
-    vcp = (struct vcache*)FILE_INODE(vmap->vm_file);
+    vcp = ITOAFS(FILE_INODE(vmap->vm_file));
     if (!vcp)
 	return;
 
@@ -350,7 +351,7 @@ void afs_linux_vma_close(struct vm_area_struct *vmap)
 
 static int afs_linux_mmap(struct file *fp, struct vm_area_struct *vmap)
 {
-    struct vcache *vcp = (struct vcache*)FILE_INODE(fp);
+    struct vcache *vcp = ITOAFS(FILE_INODE(fp));
     cred_t *credp = crref();
     struct vrequest treq;
     int code;
@@ -451,7 +452,7 @@ static int afs_linux_release(struct inode *ip, struct file *fp)
 {
     int code = 0;
     cred_t *credp = crref();
-    struct vcache *vcp = (struct vcache*)ip;
+    struct vcache *vcp = ITOAFS(ip);
 
     AFS_GLOCK();
 #ifdef AFS_LINUX24_ENV
@@ -486,7 +487,7 @@ static int afs_linux_fsync(struct file *fp, struct dentry *dp)
 #ifdef AFS_LINUX24_ENV
     lock_kernel();
 #endif
-    code = afs_fsync((struct vcache*)ip, credp);
+    code = afs_fsync(ITOAFS(ip), credp);
 #ifdef AFS_LINUX24_ENV
     unlock_kernel();
 #endif
@@ -510,7 +511,7 @@ int afs_linux_file_revalidate(kdev_t dev);
 static int afs_linux_lock(struct file *fp, int cmd, struct file_lock *flp)
 {
     int code = 0;
-    struct vcache *vcp = (struct vcache*)FILE_INODE(fp);
+    struct vcache *vcp = ITOAFS(FILE_INODE(fp));
     cred_t *credp = crref();
 #ifdef AFS_LINUX24_ENV
     struct flock64 flock;
@@ -548,7 +549,7 @@ static int afs_linux_lock(struct file *fp, int cmd, struct file_lock *flp)
  */
 int afs_linux_flush(struct file *fp)
 {
-    struct vcache *vcp = (struct vcache *)FILE_INODE(fp);
+    struct vcache *vcp = ITOAFS(FILE_INODE(fp));
     int code = 0;
     cred_t *credp;
 
@@ -652,7 +653,7 @@ static int afs_linux_revalidate(struct dentry *dp)
     int code;
     cred_t *credp;
     struct vrequest treq;
-    struct vcache *vcp = (struct vcache*)dp->d_inode;
+    struct vcache *vcp = ITOAFS(dp->d_inode);
 
     AFS_GLOCK();
 #ifdef AFS_LINUX24_ENV
@@ -705,15 +706,24 @@ static int afs_linux_dentry_revalidate(struct dentry *dp)
     struct vcache *lookupvcp = NULL;
     int code, bad_dentry = 1;
     struct sysname_info sysState;
-    struct vcache *vcp = (struct vcache*) dp->d_inode;
-    struct vcache *parentvcp = (struct vcache*) dp->d_parent->d_inode;
+    struct vcache *vcp = ITOAFS(dp->d_inode);
+    struct vcache *parentvcp = ITOAFS(dp->d_parent->d_inode);
 
     AFS_GLOCK();
 
+    sysState.allocked = 0;
+
     /* If it's a negative dentry, then there's nothing to do. */
     if (!vcp || !parentvcp)
         goto done;
 
+    /* If it is the AFS root, then there's no chance it needs 
+       revalidating */
+    if (vcp == afs_globalVp) {
+	bad_dentry = 0;
+	goto done;
+    }
+
     if (code = afs_InitReq(&treq, credp))
         goto done;
 
@@ -770,7 +780,7 @@ static int afs_linux_dentry_revalidate(struct dentry *dp)
     int code;
     cred_t *credp;
     struct vrequest treq;
-    struct inode *ip = (struct inode *)dp->d_inode;
+    struct inode *ip = AFSTOI(dp->d_inode);
     
     unsigned long timeout = 3*HZ; /* 3 seconds */
     
@@ -844,7 +854,7 @@ int afs_linux_create(struct inode *dip, struct dentry *dp, int mode)
     vattr.va_mode = mode;
 
     AFS_GLOCK();
-    code = afs_create((struct vcache*)dip, name, &vattr, NONEXCL, mode,
+    code = afs_create(ITOAFS(dip), name, &vattr, NONEXCL, mode,
 		      (struct vcache**)&ip, credp);
 
     if (!code) {
@@ -893,10 +903,10 @@ int afs_linux_lookup(struct inode *dip, struct dentry *dp)
     struct vcache *vcp=NULL;
     const char *comp = dp->d_name.name;
     AFS_GLOCK();
-    code = afs_lookup((struct vcache *)dip, comp, &vcp, credp);
+    code = afs_lookup(ITOAFS(dip), comp, &vcp, credp);
 
     if (vcp) {
-	struct inode *ip = (struct inode*)vcp;
+	struct inode *ip = AFSTOI(vcp);
 	/* Reset ops if symlink or directory. */
 #if defined(AFS_LINUX24_ENV)
        if (S_ISREG(ip->i_mode)) {
@@ -918,10 +928,10 @@ int afs_linux_lookup(struct inode *dip, struct dentry *dp)
 	else if (S_ISLNK(ip->i_mode))
 	    ip->i_op = &afs_symlink_iops;
 #endif
+    } 
     dp->d_time = jiffies;
     dp->d_op = afs_dops;
-    d_add(dp, (struct inode*)vcp);
-    } 
+    d_add(dp, AFSTOI(vcp));
 
     AFS_GUNLOCK();
     crfree(credp);
@@ -955,7 +965,7 @@ int afs_linux_link(struct dentry *olddp, struct inode *dip,
     d_drop(newdp);
 
     AFS_GLOCK();
-    code = afs_link((struct vcache*)oldip, (struct vcache*)dip, name, credp);
+    code = afs_link(ITOAFS(oldip), ITOAFS(dip), name, credp);
 
     AFS_GUNLOCK();
     crfree(credp);
@@ -970,7 +980,7 @@ int afs_linux_unlink(struct inode *dip, struct dentry *dp)
     int putback = 0;
 
     AFS_GLOCK();
-    code = afs_remove((struct vcache*)dip, name, credp);
+    code = afs_remove(ITOAFS(dip), name, credp);
     AFS_GUNLOCK();
     if (!code)
 	d_drop(dp);
@@ -994,7 +1004,7 @@ int afs_linux_symlink(struct inode *dip, struct dentry *dp,
 
     AFS_GLOCK();
     VATTR_NULL(&vattr);
-    code = afs_symlink((struct vcache*)dip, name, &vattr, target, credp);
+    code = afs_symlink(ITOAFS(dip), name, &vattr, target, credp);
     AFS_GUNLOCK();
     crfree(credp);
     return -code;
@@ -1012,7 +1022,7 @@ int afs_linux_mkdir(struct inode *dip, struct dentry *dp, int mode)
     VATTR_NULL(&vattr);
     vattr.va_mask = ATTR_MODE;
     vattr.va_mode = mode;
-    code = afs_mkdir((struct vcache*)dip, name, &vattr, &tvcp, credp);
+    code = afs_mkdir(ITOAFS(dip), name, &vattr, &tvcp, credp);
 
     if (tvcp) {
 	tvcp->v.v_op = &afs_dir_iops;
@@ -1021,7 +1031,7 @@ int afs_linux_mkdir(struct inode *dip, struct dentry *dp, int mode)
 #endif
 	dp->d_op = afs_dops;
 	dp->d_time = jiffies;
-	d_instantiate(dp, (struct inode*)tvcp);
+	d_instantiate(dp, AFSTOI(tvcp));
     }
 
     AFS_GUNLOCK();
@@ -1036,7 +1046,7 @@ int afs_linux_rmdir(struct inode *dip, struct dentry *dp)
     const char *name = dp->d_name.name;
 
     AFS_GLOCK();
-    code = afs_rmdir((struct vcache*)dip, name, credp);
+    code = afs_rmdir(ITOAFS(dip), name, credp);
 
     /* Linux likes to see ENOTEMPTY returned from an rmdir() syscall
      * that failed because a directory is not empty. So, we map
@@ -1077,7 +1087,7 @@ int afs_linux_rename(struct inode *oldip, struct dentry *olddp,
 	d_drop(newdp);
     }
     AFS_GLOCK();
-    code = afs_rename((struct vcache*)oldip, oldname, (struct vcache*)newip,
+    code = afs_rename(ITOAFS(oldip), oldname, ITOAFS(newip),
 		      newname, credp);
     AFS_GUNLOCK();
 
@@ -1105,7 +1115,7 @@ static int afs_linux_ireadlink(struct inode *ip, char *target, int maxlen,
     struct iovec iov;
 
     setup_uio(&tuio, &iov, target, 0, maxlen, UIO_READ, seg);
-    code = afs_readlink((struct vcache*)ip, &tuio, credp);
+    code = afs_readlink(ITOAFS(ip), &tuio, credp);
     crfree(credp);
 
     if (!code)
@@ -1207,7 +1217,7 @@ int afs_linux_readpage(struct file *fp, struct page *pp)
 
     setup_uio(&tuio, &iovec, (char*)address, offset, PAGESIZE,
 	      UIO_READ, AFS_UIOSYS);
-    code = afs_rdwr((struct vcache*)ip, &tuio, UIO_READ, 0, credp);
+    code = afs_rdwr(ITOAFS(ip), &tuio, UIO_READ, 0, credp);
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,0)
     unlock_kernel();
 #endif
@@ -1302,7 +1312,7 @@ int afs_linux_permission(struct inode *ip, int mode)
     if (mode & MAY_EXEC) tmp |= VEXEC;
     if (mode & MAY_READ) tmp |= VREAD;
     if (mode & MAY_WRITE) tmp |= VWRITE;
-    code = afs_access((struct vcache*)ip, tmp, credp);
+    code = afs_access(ITOAFS(ip), tmp, credp);
 
     AFS_GUNLOCK();
     crfree(credp);
@@ -1320,7 +1330,7 @@ int afs_linux_writepage_sync(struct inode *ip, struct page *pp,
                         unsigned long offset,
                         unsigned int count)
 {
-    struct vcache *vcp = (struct vcache *) ip;
+    struct vcache *vcp = ITOAFS(ip);
     char *buffer;
     loff_t base;
     int code = 0;
@@ -1373,7 +1383,7 @@ int afs_linux_updatepage(struct file *fp, struct page *pp,
 			 unsigned long offset,
 			 unsigned int count, int sync)
 {
-    struct vcache *vcp = (struct vcache *)FILE_INODE(fp);
+    struct vcache *vcp = ITOAFS(FILE_INODE(fp));
     u8 *page_addr = (u8*) afs_linux_page_address(pp);
     int code = 0;
     cred_t *credp;
