@@ -10,7 +10,7 @@
 #include <afsconfig.h>
 #include "../afs/param.h"
 
-RCSID("$Header: /cvs/openafs/src/afs/afs_call.c,v 1.14.2.7 2002/04/23 03:32:31 zacheiss Exp $");
+RCSID("$Header: /cvs/openafs/src/afs/afs_call.c,v 1.14.2.9 2002/06/08 20:29:49 shadow Exp $");
 
 #include "../afs/sysincludes.h"	/* Standard vendor system headers */
 #include "../afs/afsincludes.h"	/* Afs-based standard headers */
@@ -544,10 +544,9 @@ long parm, parm2, parm3, parm4, parm5, parm6;
 #endif
 	afs_cold_shutdown = 0;
 	if (parm == 1) afs_cold_shutdown = 1;
-	if (afs_globalVFS != 0) {
-	    afs_warn("AFS isn't unmounted yet! Call aborted\n");
-	    code = EACCES;
-	}
+	if (afs_globalVFS != 0) 
+	    afs_warn("AFS isn't unmounted yet!\n");
+
 	afs_shutdown();
     }
 
